(function(){

	/*
	 * This function does the interesting work,  We find
	 * all Lenovo computers for which a, OEM serial number
	 * is known, but no warranty date is known.  For each
	 * such computer, we send a request to Lenovo for the
	 * information.  The status of each request is written
	 * to the console.
	 *
	 * The warranty date is updated on success.
	 */
	app.progress.show();

	var success = true,
		count = 0, warrants = 0,
		lenovos = app.selection;
	if (!lenovos || !lenovos.length) {
		lenovos = new KSView("computer");
		lenovos.select("('lenovo'~=manu)&&(oeid!=NULL)");
	}

	if (!lenovos.length) {
		console.info("no computers match the requirements");
		return;
	}

	lenovos.forEach(function (obj) {
		var sn = obj.biosserial || obj.oemserial;

		if (!sn || !obj.manufacturer || (obj.manufacturer.toLowerCase().indexOf('lenovo') < 0))
			return(false);

		app.progress.comment("requesting warranty date for " + sn);
		console.info("requesting warranty date for " + sn);

		var xhr = new XMLHttpRequest();
		xhr.setRequestHeader("Accept", "application/json");
		xhr.open("GET", "https://pcsupport.lenovo.com/us/en/api/warranty/WarrantyLookup/GetWarrantyAndRepair?productid=" + sn, false);
		xhr.send();

		if (xhr.status == 401) {
			console.error("access denied (" + xhr.statusText + ")");
			success = false;
			return(true);	/* access denied, stop processing */
		}

		count++;
		if (xhr.status != 200) {
			console.warn("no results for " + sn + " (" + xhr.statusText + ")");
		} else {
			var results = xhr.response,
				adate = results && results.Warranty && results.Warranty.EntireWarrantyPeriod && results.Warranty.EntireWarrantyPeriod.End;
			if (!adate) {
				console.warn("no warranty date found for " + sn);
			} else {
				adate = new Date(adate);
				adate = new Date(adate.getUTCFullYear(), adate.getUTCMonth(), adate.getUTCDate(), 12, 0, 0, 0);
				console.info("got warranty date for " + sn + ": " + adate);
				obj.warrantydate = adate;
				warrants++;

				obj.commit();
			}
		}
	});

	console.info("processed " + count + " computers (" + lenovos.length + " candidates)");
	if (warrants)
		console.info("set warranty date on " + warrants + " computers");

})();
